package parkomat.tests.model;

import static org.junit.Assert.*;

import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import parkomat.model.*;
import parkomat.model.PagamentoAttivo;

public class TariffaTest
{

	private List<FasciaTariffaria> fasceFeriali;
	private Map<DayOfWeek, List<FasciaTariffaria>> mappaFasce, mappaFasceSenzaGiovedi, mappaFasceConMartediBucato;
	
	@Before
	public void setUp() {
		/*
		Tariffa H1
		costo 0.50, franchigia 0, minimo 60
		Lunedì,	  8-20
		Martedì,  8-20
		Mercoledì,8-20
		Giovedì,  8-20
		Venerdì,  8-20
		Sabato,	  8-20
		NON completata sulle le fasce mancanti gratuite
		 */
		fasceFeriali = List.of(
				new FasciaTariffaria(LocalTime.of(0,0), LocalTime.of(8,0), PagamentoAttivo.GRATUITO),
				new FasciaTariffaria(LocalTime.of(8,0), LocalTime.of(20,0), PagamentoAttivo.A_PAGAMENTO),
				new FasciaTariffaria(LocalTime.of(20,0), LocalTime.of(0,0), PagamentoAttivo.GRATUITO)
				); 
		mappaFasce = Map.of(
				DayOfWeek.MONDAY, fasceFeriali,
				DayOfWeek.TUESDAY, fasceFeriali,
				DayOfWeek.WEDNESDAY, fasceFeriali,
				DayOfWeek.THURSDAY, fasceFeriali,
				DayOfWeek.FRIDAY, fasceFeriali,
				DayOfWeek.SATURDAY, fasceFeriali,
				DayOfWeek.SUNDAY, fasceFeriali				
				);

		mappaFasceSenzaGiovedi = Map.of(
				DayOfWeek.MONDAY, fasceFeriali,
				DayOfWeek.TUESDAY, fasceFeriali,
				DayOfWeek.WEDNESDAY, fasceFeriali,
				// DayOfWeek.THURSDAY, fasceFeriali, // manca giovedì
				DayOfWeek.FRIDAY, fasceFeriali,
				DayOfWeek.SATURDAY, fasceFeriali,
				DayOfWeek.SUNDAY, fasceFeriali				
				);
		
		mappaFasceConMartediBucato = Map.of(
				DayOfWeek.MONDAY, fasceFeriali,
				DayOfWeek.TUESDAY, List.of(new FasciaTariffaria(LocalTime.of(8,0), LocalTime.of(20,0),PagamentoAttivo.A_PAGAMENTO)),
				DayOfWeek.WEDNESDAY, fasceFeriali,
				DayOfWeek.THURSDAY, fasceFeriali, 
				DayOfWeek.FRIDAY, fasceFeriali,
				DayOfWeek.SATURDAY, fasceFeriali,
				DayOfWeek.SUNDAY, fasceFeriali				
				);
	}
	

	@Test
	public void testInteraSettimanaTariffaCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasce);
		assertEquals("H1", t.getNomeTariffa());
		assertTrue(t.isValid());
	}

	@Test
	public void testSoloLunediTariffaCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasce);
		assertEquals("H1", t.getNomeTariffa());
		assertTrue(t.isValid(DayOfWeek.MONDAY));
	}

	@Test
	public void testTuttiGiorniTariffaCostruitaConUnaFasciaAllaVolta()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60);
		for (DayOfWeek day : DayOfWeek.values()) {
			for (FasciaTariffaria f: fasceFeriali) t.addFascia(f, day);
			assertTrue(t.isValid(day));
		}
		assertEquals("H1", t.getNomeTariffa());
	}

	@Test
	public void testTuttiGiorniTariffaCostruitaConUnaFasciaAllaVoltaARitroso()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60);
		FasciaTariffaria f = null;
		for (DayOfWeek day : DayOfWeek.values()) {
			for (ListIterator<FasciaTariffaria> it = fasceFeriali.listIterator(fasceFeriali.size()); it.hasPrevious(); ) {
				f = it.previous();
				t.addFascia(f, day);
			}
			// System.out.println(t);
			assertTrue(t.isValid(day));
		}
		assertEquals("H1", t.getNomeTariffa());
	}
	
	// casi che devono fallire
	
	@Test // la lista del giovedì è null, ma isValid cattura la situazione e risponde semplicemente false
	public void testInteraSettimanaTariffaSenzaGiovediCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasceSenzaGiovedi);
		assertEquals("H1", t.getNomeTariffa());
		assertFalse(t.isValid());
	}

	@Test // la lista del martedì non è null, ma non è valida perché non copre le 24 ore
	public void testInteraSettimanaTariffaBucataMartediCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasceConMartediBucato);
		assertEquals("H1", t.getNomeTariffa());
		assertFalse(t.isValid());
	}

}
